/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dcfilebrowser.h"
#include "config.h"
#include <QApplication>
#include <QLabel>
#include <QCursor>
#include <QFileDialog>
#include <QUrl>
#include <QToolButton>
#include <QComboBox>
#include <QClipboard>
#include <QMessageBox>
#include <QStyle>
#include <QMdiArea>
#include <QList>
#include <QInputDialog>
#include <QtAlgorithms>
#include <QColor>
#include <QtConcurrentFilter>

#include <QtDebug>

#include <set>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include "dcmenuhandler.h"
#include "dcconfig.h"
#include "dciconloader.h"
#include "dcfilelistdecompressor.h"
#include "dcevent.h"

// for user menu commands
#include "dcclient.h"
#include "dcconnectionmanager.h"

// for searching by TTH
#include "dchubsearch.h"

#include "dcfiletool.h"

#include <dclib/dcos.h>
#include <dclib/core/cbytearray.h>
#include <dclib/core/cdir.h>
#include <dclib/core/cxml.h>

// for file type icons
#include <dclib/cfilemanager.h>

#include "dcguiutils.h"
#include "filebrowsermodel.h"

DCFileBrowser::QStringHash DCFileBrowser::download_dirs;

/** */
DCFileBrowser::DCFileBrowser(QWidget * parent, bool allowopen)
: QWidget(parent) {
    setupUi(this);

    m_bAllowOpen = allowopen;
    m_bAllowDownload = true;

    // set caption
    setWindowTitle(tr("Filebrowser"));
    setWindowIcon(DCIconLoader::Instance()->GetPixmap(eiVIEW_SIDETREE));

    model = new FileBrowserModel(this);
    model->setIconsScaled(true, QSize(16, 16));

    treeModel = new FileBrowserModel(this);
    treeModel->setIconsScaled(true, QSize(16,16));

    rootElem = NULL;
    proxyRoot = NULL;

    m_nShareSize = 0;
    m_nFileCount = 0;

    m_pDecomp = 0;
    m_pListData = 0;

    ToolButton_OPEN->setIcon(DCIconLoader::Instance()->GetPixmap(eiOPEN));
    ToolButton_SAVE->setIcon(DCIconLoader::Instance()->GetPixmap(eiSAVE));
    ToolButton_GOTO_USER->setIcon(DCIconLoader::Instance()->GetPixmap(eiUSERS));

    toolButton_FORW->setEnabled(false);
    toolButton_BACK->setEnabled(false);
    toolButton_UP->setEnabled(false);
    toolButton_FORW->setIcon(DCIconLoader::Instance()->GetPixmap(eiGO_NEXT));
    toolButton_BACK->setIcon(DCIconLoader::Instance()->GetPixmap(eiGO_PREVIOUS));
    toolButton_UP->setIcon(DCIconLoader::Instance()->GetPixmap(eiGO_UP));

    history = DCFileBrowserHistory<FileBrowserItem*>(15);

    connect(toolButton_FORW, SIGNAL(clicked()), this, SLOT(slotHistoryForward()));
    connect(toolButton_BACK, SIGNAL(clicked()), this, SLOT(slotHistoryBack()));
    connect(toolButton_UP, SIGNAL(clicked()), this, SLOT(slotDirUp()));

    connect(treeView_DIRECTORY, SIGNAL(clicked(const QModelIndex&)), this, SLOT(slotCurrentChangedDirectory(const QModelIndex &)));
    connect(treeView_DIRECTORY, SIGNAL(customContextMenuRequested(const QPoint &)),
            this, SLOT(slotDIRECTORY_customContextMenuRequested(const QPoint &)));
    connect(treeView_FILES, SIGNAL(customContextMenuRequested(const QPoint &)),
            this, SLOT(slotFILES_customContextMenuRequested(const QPoint &)));
    connect(treeView_FILES, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(slotItemDoubleClicked(const QModelIndex&)));
    connect(treeView_SEARCH, SIGNAL(clicked(const QModelIndex&)), this, SLOT(slotSearchResultClicked(QModelIndex)));

    connect(ToolButton_OPEN, SIGNAL(clicked()), this, SLOT(slotFileOpen()));
    connect(ToolButton_SAVE, SIGNAL(clicked()), this, SLOT(slotFileSave()));
    connect(ToolButton_GOTO_USER, SIGNAL(clicked()), this, SLOT(slotGotoUser()));

    ToolButton_OPEN->setEnabled(m_bAllowOpen);
    ToolButton_GOTO_USER->setEnabled(false);

    setAttribute(Qt::WA_DeleteOnClose);

    treeView_SEARCH->hide();

    m_pContainer = NULL;
}

/** */
DCFileBrowser::~DCFileBrowser() {

    treeView_DIRECTORY->setUpdatesEnabled(false);
    treeView_FILES->setUpdatesEnabled(false);

    delete m_pDecomp;
    delete m_pListData;

    model->setRootElem(NULL, false, false);//reset root element
    delete model;

    treeModel->setRootElem(NULL, false, false);//reset root element
    delete treeModel;

    delete rootElem;

    delete proxyRoot;

    search_engine->reset();

    delete search_engine;
}

/** */
void DCFileBrowser::customEvent(QEvent * event) {
    if (event->type() == EVENT_OPEN_FILELIST_FOLDER) {
/*DCFileBrowserListItem * item = ((DC_OpenFilelistFolderEvent*) event)->m_pItem;

        treeView_DIRECTORY->clearSelection();
        treeView_DIRECTORY->ensureItemVisible(item);
        treeView_DIRECTORY->setSelected(item, true);
        treeView_DIRECTORY->setCurrentItem(item);*/

        event->accept();
    } else {
        event->ignore();
    }
}

/** */
bool DCFileBrowser::eventFilter(QObject * object, QEvent * event){
    if (event->type() == QEvent::KeyRelease){
        QKeyEvent *k_e = (QKeyEvent*)event;

        if (k_e->key() == Qt::Key_Enter || k_e->key() == Qt::Key_Return){
            if ((QTreeView*)object == treeView_FILES){
                QModelIndexList selected = treeView_FILES->selectionModel()->selection().indexes();

                if (!selected.isEmpty())
                    slotItemDoubleClicked(selected.at(0));
            }
            else if ((QLineEdit*)object == lineEdit_PATH){
                QString path = lineEdit_PATH->text();

                FileBrowserItem *item = NULL;

                if (path.indexOf(".") == 0){//check for relative path
                    path.replace(".", "");

                    item = model->createRootForPath(path);
                }
                else
                    item = model->createRootForPath(path, rootElem);

                if (item)
                    model->setRootElem(item, false, true);
            }
            else if ((QLineEdit*)object == lineEdit_SEARCHSTR){
                if (!search_engine->isRunning()){
                    slotSearchReset();
                    slotDoSearch();
                }
            }
        }
        else if (k_e->key() == Qt::Key_Backspace){
            if ((QTreeView*)object == treeView_FILES){
                slotDirUp();
            }
        }

    }

    return QWidget::eventFilter(object, event);
}

/** */
void DCFileBrowser::slotThreadFinishedLoading() {
    /* name may have been updated */
    m_sFileName = QString::fromAscii(m_pDecomp->getFileName().Data());

    CXml * xml = m_pDecomp->takeXml();
    if (xml) {
        CByteArray * ba = m_pDecomp->takeCompressed();
        if (ba) {
            delete m_pListData;
            m_pListData = ba;
        }

        ContinueInitTree(xml, NULL);
        delete xml;
    } else {
        CString * text = m_pDecomp->takeText();
        if (text) {
            CByteArray * ba = m_pDecomp->takeCompressed();
            if (ba) {
                delete m_pListData;
                m_pListData = ba;
            }

            ContinueInitTree(NULL, text);
            delete text;
        } else {
            QMessageBox::critical(
                    this,
                    tr("Failed to load filelist"),
                    tr("Unable to load") + " " + m_sFileName
                    );
            setWindowTitle(tr("Filebrowser"));
            EnableGuiElements();
        }
    }

    disconnect(m_pDecomp, SIGNAL(finished()), this, SLOT(slotThreadFinishedLoading()));
    delete m_pDecomp;
    m_pDecomp = NULL;
}

/** */
void DCFileBrowser::slotFileOpen() {
    QString file = QString::fromAscii(DCConfig::Instance()->GetFileListPath().Data());

    QString s = QFileDialog::getOpenFileName(
            this,
            tr("Choose a file"),
            file,
            tr("Modern XML Filelists") + " (*.xml.bz2);;" +
            tr("Modern XML Filelists (uncompressed)") + " (*.xml);;" +
            tr("Old BZ2 Text Filelists") + " (*.bz2);;" +
            tr("Very old HE3 Text Filelists") + " (*.DcLst);;" +
            tr("Old Valknut Filelists") + " (*.filelist);;" +
            tr("All files") + " (*)"
            );

    if (!s.isEmpty()) {
        model->setRootElem(new FileBrowserItem(QList<QVariant>(), NULL), false, false);
        treeModel->setRootElem(new FileBrowserItem(QList<QVariant>(), NULL), false, false);

        delete rootElem;
        rootElem = NULL;

        InitTree(QString::null, QString::null, QString::null, s);
    }
}

/** */
void DCFileBrowser::slotFileSave() {
    if ((m_pListData == NULL) || (m_pListData->Size() == 0)) {
        QMessageBox::critical(
                this,
                tr("Cannot save file"),
                tr("No data available to save.")
                );
        return;
    }

    QString s = QFileDialog::getSaveFileName(
            this,
            tr("Choose a filename to save under"),
            m_sFileName
            );

    if (s.isEmpty()) {
        return;
    }

    if (m_pListData->SaveToFile(s.toAscii().constData()) == false) {
        QMessageBox::critical(
                this,
                tr("Error saving file"),
                tr("Failed to save") + " " + s
                );
    }
}

/** */
void DCFileBrowser::slotCurrentChangedDirectory(const QModelIndex &index) {
    if (!index.isValid())
        return;

    FileBrowserItem *item = static_cast<FileBrowserItem*>(index.internalPointer());

    if (!item || !item->parent())
        return;

    QString path = treeModel->createRemotePath(item);

    FileBrowserItem *new_root = model->createRootForPath(path, rootElem);

    if (!new_root)
        return;

    model->setRootElem(new_root, false, true);
}

/** */
void DCFileBrowser::AddFile(FileBrowserItem * item, FileBrowserItem * fileitem) {
    // increment filecounter
    if (!fileitem->isDir) {
        m_nFileCount++;
        m_nShareSize += fileitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong();
    }

    // add fileitem to filelist
    item->appendChild(fileitem);

    if (!item->parent())//do not update header
        return;

    // add to folder contents size
    qulonglong new_size = item->data(COLUMN_FILEBROWSER_ESIZE).toULongLong()+fileitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong();
    item->updateColumn(COLUMN_FILEBROWSER_ESIZE, new_size);
    item->updateColumn(COLUMN_FILEBROWSER_SIZE, DCGuiUtils::GetSizeString(new_size));
}

/** */
void DCFileBrowser::InitTree(QString nick, QString hubname, QString hubhost, QString filename, QString jumpto, QStringList dirs) {
    if (filename.isEmpty()) {
        return;
    }

    if (m_pDecomp != 0) {
        QMessageBox::critical(
                this,
                tr("Cannot open list now"),
                tr("Filelist browser is busy")
                );
        return;
    }

    // store values
    m_sNick = nick;
    m_sHubName = hubname;
    m_sHubHost = hubhost;
    m_sJumpTo = jumpto;
    m_lDownloadDirs = dirs;

    // first check if filename is our own list
    if (filename == "/dev/null/ownfilelist") {
        m_bAllowDownload = false;
        m_sFileName = "Own Filelist"; /* temp value */

        DisableGuiElements();
        m_pDecomp = new DCFilelistDecompressor(this, CString());
    } else {
        m_bAllowDownload = true;
        m_sFileName = filename;

        if (!QFile::exists(filename)) {
            QMessageBox::critical(
                    this,
                    tr("Error opening filelist"),
                    tr("File") + " " + filename + " " + tr("does not exist.")
                    );

            return;
        }

        DisableGuiElements();
        m_pDecomp = new DCFilelistDecompressor(this, filename.toAscii().constData());
    }

    connect(m_pDecomp, SIGNAL(finished()), this, SLOT(slotThreadFinishedLoading()));
    m_pDecomp->start();
}

/** */
void DCFileBrowser::ContinueInitTree(CXml * xml, CString * text) {
    /* A bz2 file that decompressed to zero bytes probably... */
    if ((xml == NULL) && ((text == NULL) || (text->Length() == 0))) {
        QMessageBox::critical(
                this,
                tr("Error opening filelist"),
                m_sFileName + " " + tr("is likely not a valid filelist")
                );
        setWindowTitle(tr("Filebrowser"));
        EnableGuiElements();
        return;
    }

    // reset count values
    m_nShareSize = 0;
    m_nFileCount = 0;

    // disable updates
    treeView_DIRECTORY->setUpdatesEnabled(false);

    // parse filename

    QFileInfo fi(m_sFileName);
    QString fname = fi.fileName();

    if ((m_sNick.isEmpty()) || (m_sHubName.isEmpty()) || (m_sHubHost.isEmpty())) {
        int i = fname.indexOf('@');
        if (i != -1) {
            m_sNick = fname.left(i);
            m_sHubHost = fname.mid(i + 1);
            m_sHubHost.remove(".xml.bz2");
            m_sHubHost.remove(".bz2");
            m_sHubHost.remove(".xml");
            m_sHubHost.remove(".DcLst");
            m_sHubHost.remove(".filelist");

            // replace _ with : before port number
            int i2 = m_sHubHost.lastIndexOf('_');
            if (i2 != -1) {
                bool ok;
                m_sHubHost.mid(i2 + 1).toInt(&ok);
                if (ok) {
                    m_sHubHost = m_sHubHost.left(i2) + ":" + m_sHubHost.mid(i2 + 1);
                }
            }

            if (m_sHubName.isEmpty()) {
                m_sHubName = m_sHubHost;
            }
        }
    }

    // set dialog caption

    if (m_sNick.isEmpty()) {
        fname.remove(".xml.bz2");
        fname.remove(".bz2");
        fname.remove(".xml");
        fname.remove(".DcLst");
        fname.remove(".filelist");
        m_sNick = fname;
    }

    if (m_sHubName.isEmpty()) {
        setWindowTitle(m_sNick + " - " + tr("Filebrowser"));
    } else {
        setWindowTitle(m_sNick + " - " + tr("Filebrowser") + " [" + m_sHubName + "]");
    }

    // create root item
    QList<QVariant> rootData;
    rootData << tr("Name") << tr("Size") << tr("Exact size") << tr("Type") << tr("TTH");
    rootElem = new FileBrowserItem(rootData, NULL);

    proxyRoot = new FileBrowserItem(rootData, NULL);

    if (xml) {
        InitXMLTree(rootElem, xml);
    } else {
        InitTXTTree(rootElem, text);
    }

    // update summary
    TextLabel_SUMMARY->setText(QString().setNum(m_nFileCount) + " " + tr("Files") + " (" +
            DCGuiUtils::GetSizeString(m_nShareSize) + " " + tr("Total") + ")");

    // update tooltip
    TextLabel_SUMMARY->setToolTip(QString().setNum(m_nShareSize) + " B");

    // update view
    treeView_DIRECTORY->setUpdatesEnabled(true);
//    treeView_DIRECTORY->triggerUpdate();

    // open first tree
//    treeView_DIRECTORY->setOpen(treeView_DIRECTORY->firstChild(), true);

    EnableGuiElements();

    model->setRootElem(rootElem, true, true);
    connect(model, SIGNAL(rootChanged(FileBrowserItem*,FileBrowserItem*)),
            this, SLOT(slotRootChanged(FileBrowserItem*,FileBrowserItem*)));
    lineEdit_PATH->installEventFilter(this);
    lineEdit_SEARCHSTR->installEventFilter(this);

    foreach(FileBrowserItem *i, rootElem->childItems){
        if (i->isDir)
            createProxyTree(proxyRoot, i);
    }

    treeModel->setRootElem(proxyRoot, true, true);
    treeModel->sort(COLUMN_FILEBROWSER_NAME, Qt::AscendingOrder);

    treeView_DIRECTORY->setModel(treeModel);
    treeView_DIRECTORY->header()->hideSection(COLUMN_FILEBROWSER_ESIZE);
    treeView_DIRECTORY->header()->hideSection(COLUMN_FILEBROWSER_TTH);
    treeView_DIRECTORY->header()->hideSection(COLUMN_FILEBROWSER_TYPE);
    treeView_DIRECTORY->setColumnWidth(COLUMN_FILEBROWSER_NAME, 150);
    treeView_DIRECTORY->setColumnWidth(COLUMN_FILEBROWSER_SIZE, 150);

    Frame6->resize(310, Frame6->size().height());

    treeView_FILES->setModel(model);
    treeView_FILES->setAlternatingRowColors(DCConfig::Instance()->GetAlternatingRowColors());
    treeView_FILES->setColumnWidth(COLUMN_FILEBROWSER_NAME, 400);
    treeView_FILES->setColumnWidth(COLUMN_FILEBROWSER_SIZE, 80);
    treeView_FILES->setColumnWidth(COLUMN_FILEBROWSER_ESIZE, 120);
    treeView_FILES->setColumnWidth(COLUMN_FILEBROWSER_TYPE, 70);

    /* jump to target file or folder */
    if (!m_sJumpTo.isEmpty() && !m_sJumpTo.isNull()) {
        JumpTo(m_sJumpTo);
    }

    search_engine = new DCFileBrowserSearchEngine(model);
    connect(search_engine, SIGNAL(modelCreated(FileBrowserModel*)),
            this, SLOT(slotNewModel(FileBrowserModel*)));
    connect(search_engine, SIGNAL(stateChanged(DCFileBrowserSearchEngine::State)),
            this, SLOT(slotSearchEngineStateChanged(DCFileBrowserSearchEngine::State)));
    connect(pushButton_STARTSEARCH, SIGNAL(clicked()), this, SLOT(slotDoSearch()));
    connect(pushButton_SEARCHRESET, SIGNAL(clicked()), this, SLOT(slotSearchReset()));

    /* download any directories */
    for (QStringList::const_iterator it = m_lDownloadDirs.constBegin(); it != m_lDownloadDirs.constEnd(); ++it) {
        DownloadDirectory(*it);
    }
    m_lDownloadDirs.clear();

    treeView_FILES->model()->sort(COLUMN_FILEBROWSER_NAME, Qt::AscendingOrder);

    label_SIZE->hide();

    connect(treeView_FILES->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)),
            this, SLOT(selectionChanged(QItemSelection,QItemSelection)));

    connect(treeView_DIRECTORY, SIGNAL(activated(const QModelIndex&)), this,
            SLOT(slotCurrentChangedDirectory(const QModelIndex &)));

    treeView_FILES->installEventFilter(this);

#if HAVE_MALLOC_TRIM
    malloc_trim(0);
#endif
}

void DCFileBrowser::createProxyTree(FileBrowserItem *root, FileBrowserItem *item){
    if (!root || !item)
        return;

    if (!item->isDir)
        return;

    FileBrowserItem *child = new FileBrowserItem(QList<QVariant>(), root);
    *child = *item;

    foreach (FileBrowserItem *i, item->childItems){
        if (i->isDir)
            createProxyTree(child, i);
    }

    root->appendChild(child);
}

/** */
void DCFileBrowser::DisableGuiElements() {
    setWindowTitle(tr("Processing filelist..."));
    ToolButton_OPEN->setEnabled(false);
    ToolButton_SAVE->setEnabled(false);
    ToolButton_GOTO_USER->setEnabled(false);
    treeView_DIRECTORY->setEnabled(false);
    treeView_FILES->setEnabled(false);
    lineEdit_SEARCHSTR->setEnabled(false);
    comboBox_SEARCHTYPE->setEnabled(false);
    pushButton_SEARCHRESET->setEnabled(false);
    pushButton_STARTSEARCH->setEnabled(false);
}

void DCFileBrowser::EnableGuiElements() {
    ToolButton_OPEN->setEnabled(m_bAllowOpen);
    ToolButton_SAVE->setEnabled(true);
    treeView_DIRECTORY->setEnabled(true);
    treeView_FILES->setEnabled(true);
    lineEdit_SEARCHSTR->setEnabled(true);
    comboBox_SEARCHTYPE->setEnabled(true);
    pushButton_SEARCHRESET->setEnabled(true);
    pushButton_STARTSEARCH->setEnabled(true);

    if (!m_bAllowOpen || m_sNick.isEmpty() || m_sHubHost.isEmpty()) {
        ToolButton_GOTO_USER->setEnabled(false);
    } else {
        ToolButton_GOTO_USER->setEnabled(true);
    }
}

/** */
void DCFileBrowser::InitTXTTree(FileBrowserItem * item, CString * data) {
    qDebug() << "DCFileBrowser::InitTXTTree";
    QString s1, sname, slength;
    long i, j, d;
    int depth;

    i = j = 0;

    while ((i = data->Find("\xD\xA", j)) >= 0) {
        s1 = data->Mid(j, i - j).Data();

        depth = 0;
        while ((s1.indexOf("\x9", depth)) != -1)
            depth++;
        s1.remove("\x9");

        if (item)
            while (depth < item->row())
                if ((item = item->parent()) == 0)
                    break;

        if (s1.isEmpty())
            s1 = "\\";

        if (!s1.isEmpty()) {
            d = s1.lastIndexOf("|");

            if (d != -1) // parse file entry
            {
                qulonglong size = s1.right(s1.length() - d - 1).toULongLong();
                FileBrowserItem * FileItem = new FileBrowserItem(QList<QVariant>() << s1.mid(0, d)
                                                                                   << DCGuiUtils::GetSizeString(size)
                                                                                   << size
                                                                                   << tr("File")
                                                                                   << "",
                                                                                   item);
                FileItem->isDir = false;

                AddFile(item, FileItem);
            } else // parse dir entry
            {
                FileBrowserItem * FileItem = new FileBrowserItem(QList<QVariant>() << s1
                                                                                   << ""
                                                                                   << 0
                                                                                   << tr("Directory")
                                                                                   << "",
                                                                                   item);

                FileItem->isDir = true;

                AddFile(item, FileItem);
            }
        }

        j = i + 2;
    }

    // fixup the folder sizes
    FileBrowserItem * cur = (rootElem->childItems.size()>0)?rootElem->childItems.at(0):NULL;
    while (cur) {
        CalcDirSize(cur);

        cur = cur->nextSibling();
    }

}

/** */
void DCFileBrowser::InitXMLTree(FileBrowserItem * item, CXml * xml) {
    if (xml && xml->DocFirstChild()) {
        // maybe we should just parse the first FileListing?
        do {
            if ((xml->Name() == "FileListing") && xml->FirstChild()) {
                ParseXMLTree(xml, item);
                xml->Parent();
            }
        } while (xml->NextNode());
    }
}

/** */
void DCFileBrowser::ParseXMLTree(CXml * xml, FileBrowserItem * item) {
    CString s;

    do {
        if (xml->Name() == "File") {
            qulonglong size = xml->Prop("Size").asULL();
            FileBrowserItem * FileItem = new FileBrowserItem(QList<QVariant>() << QString(xml->Prop("Name").Data())
                                                                               << DCGuiUtils::GetSizeString(size)
                                                                               << size
                                                                               << tr("File")
                                                                               << QString(xml->Prop("TTH").Data()),
                                                                               item);

            FileItem->isDir = false;
            FileItem->type = CFileManager::Instance()->GetFileType(xml->Prop("Name"));

            AddFile(item, FileItem);
        } else if (xml->Name() == "Directory") {
            FileBrowserItem * item1 = new FileBrowserItem(QList<QVariant>() << QString(xml->Prop("Name").Data())
                                                                            << ""
                                                                            << 0
                                                                            << tr("Directory")
                                                                            << "",
                                                                            item);

            item1->isDir = true;

            if (xml->FirstChild()) {
                ParseXMLTree(xml, item1);
                xml->Parent();
            }

            AddFile(item, item1);
        }
    } while (xml->NextNode());
}

static bool onlyNameColumn(const QModelIndex &index){
    return (index.column() == COLUMN_FILEBROWSER_NAME);
}

static void addToList(QModelIndexList &list, const QModelIndex &index){
     list.append(index);
}

void DCFileBrowser::selectionChanged(const QItemSelection &, const QItemSelection &){
    QItemSelectionModel *selection_model = treeView_FILES->selectionModel();
    QModelIndexList all_selected = selection_model->selection().indexes();
    QFuture<QModelIndexList> filtered = QtConcurrent::filteredReduced(all_selected, onlyNameColumn, addToList);
    QModelIndexList selected = filtered.result();

    qulonglong size = 0;

    foreach (QModelIndex index, selected){
        FileBrowserItem *item = static_cast<FileBrowserItem*>(index.internalPointer());

        if (!item)
            continue;

        size += item->data(COLUMN_FILEBROWSER_ESIZE).toULongLong();
    }

    if (size == 0){
        label_SIZE->hide();
    }
    else{
        QString text_size = DCGuiUtils::GetSizeString(size);

        label_SIZE->setText(tr("Total size: ") + text_size);
        label_SIZE->show();
    }
}

void DCFileBrowser::slotSearchResultClicked(const QModelIndex &index){

    if (!index.isValid())
        return;

    FileBrowserItem *item = (FileBrowserItem*)index.internalPointer();

    if (!item)
        return;

    FileBrowserItem *new_root = search_engine->mapToOldModel(item);

    if (new_root){
        if (!new_root->isDir){
            new_root = new_root->parent();

            if (!new_root)
                return;
        }
        model->setRootElem(new_root, false, true);
    }
}

/** */
void DCFileBrowser::CreateMenu(const QPoint &, bool direntry) {
    QString s;
    qulonglong size = 0;
    QString rootPath = QString::null;
    QString localName = QString::null;
    QItemSelectionModel *selection_model;

    if (!direntry)
        selection_model = treeView_FILES->selectionModel();
    else
        selection_model = treeView_DIRECTORY->selectionModel();

    QModelIndexList all_selected = selection_model->selection().indexes();
    QFuture<QModelIndexList> filtered = QtConcurrent::filteredReduced(all_selected, onlyNameColumn, addToList);
    QModelIndexList selected = filtered.result();

    FileBrowserItem *curitem;

    bool hasFolderItems = false;
    DCClient * client = 0;

    QMap<QAction*, DC_UserMenuCommand*> addedcommands;

    if (direntry) {

        QModelIndexList mapped;
        FileBrowserItem *old_root = model->getRootElem();

        //fucking hack
        //root elements may be not same in models
        model->blockSignals(true);

        model->setRootElem(rootElem, false, true);

        foreach (QModelIndex ind, selected){
            FileBrowserItem *proxyItem = static_cast<FileBrowserItem*>(ind.internalPointer());

            if (!proxyItem)
                return;

            QString path = treeModel->createRemotePath(proxyItem);
            FileBrowserItem *new_item = model->createRootForPath(path, rootElem);
            QModelIndex index = model->createIndexForItem(new_item);

            if (!index.isValid() || !new_item)
                continue;

            mapped.append(index);
        }

        selected.clear();

        selected.append(mapped);

        model->setRootElem(old_root, false, true);

        model->blockSignals(false);
    } else {
        for (int j = 0; j < selected.size(); j++){
            FileBrowserItem *item = (FileBrowserItem*)selected.at(j).internalPointer();

            if (item && item->isDir){
                hasFolderItems = true;

                break;
            }
        }
    }
    const int i = selected.size();

    // nothing on no selection
    if (i == 0)
        return;

    curitem = (FileBrowserItem*)selected.at(0).internalPointer();

    QMenu m(this);

    QAction * dl = DCMenuHandler::addAction(&m, emiDOWNLOAD, m_bAllowDownload);
    QMenu * dl_to = DCMenuHandler::addMenu(&m, emiDOWNLOAD_TO, m_bAllowDownload, tr("Download to"));
    // disable 'download as' on directories
    QAction * dl_as = DCMenuHandler::addAction(&m, emiDOWNLOAD_AS, (m_bAllowDownload && !direntry && !hasFolderItems && (i == 1)));
    QAction * dl_in = DCMenuHandler::addAction(&m, emiDOWNLOAD_IN, (m_bAllowDownload && !direntry && !hasFolderItems));
    DCMenuHandler::addAction(&m, emiSEPARATOR);
    QAction * search_tth = m.addAction(QIcon(DCIconLoader::Instance()->GetPixmap(eiFILEFIND)), tr("Search by TTH"));
    search_tth->setEnabled(m_bAllowOpen && !direntry && curitem && !(curitem->data(COLUMN_FILEBROWSER_TTH).toString().isEmpty()));
    DCMenuHandler::addAction(&m, emiSEPARATOR);
    QAction * copymagnet = DCMenuHandler::addAction(&m, emiCOPYMAGNETLINK, (!direntry && !hasFolderItems));

    client = DCConnectionManager::Instance()->GetClientForHub(m_sHubName.toAscii().constData(), m_sHubHost.toAscii().constData());

    if (client != 0)
        addedcommands = client->AddMenuCommands(&m, euccFilelist);

    QList<DCDownloadAlias*> alias_list = DCConfig::Instance()->GetDownloadAliases();
    QList<QAction*> d_to_actions;

    for (int i = 0; i < alias_list.size(); i++) {
        DCDownloadAlias* cur = alias_list.at(i);
        QAction *t = DCMenuHandler::addAction(dl_to, emiDOWNLOAD_FOLDER, true, cur->alias);
        t->setToolTip(cur->path);
        t->setText(cur->alias);

        d_to_actions << t;
    }

    dl_to->addSeparator();

    if (!DCFileBrowser::GetDownloadDirs().isEmpty()){
        QStringHash &hash = DCFileBrowser::GetDownloadDirs();
        QHashIterator<QString, QString> it(hash);

        while (it.hasNext()){
            it.next();

            QAction *t = DCMenuHandler::addAction(dl_to, emiDOWNLOAD_FOLDER, true);
            t->setToolTip(it.key());
            t->setText(it.value());

            d_to_actions << t;
        }
    }

    QAction *dl_to_browse = DCMenuHandler::addAction(dl_to, emiDOWNLOAD_AS, true, tr("Browse"));
    dl_to_browse->setText(tr("Browse"));

    QAction * chosen = m.exec(QCursor::pos());
    QString t_tip = "";

    if (d_to_actions.contains(chosen))
        t_tip = chosen->toolTip();

    if (chosen == copymagnet) {
        //magnet:?xt=urn:tree:tiger:EOSA5AGTL5SD3VWCF3R2OH2WMGXV3S3R7SYN4YA&xl=708780032&dn=FC-6-i386-disc1.iso
        QString all;

        for (int c = 0; c < selected.size(); c++) {

            curitem = (FileBrowserItem*)selected.at(c).internalPointer();

            if (!curitem)
                continue;

            // add tth
            all += "magnet:?xt=urn:tree:tiger:";
            all += curitem->data(COLUMN_FILEBROWSER_TTH).toString();

            // add filesize
            all += "&xl=";
            all += QString().setNum(curitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong());

            // add filename
            all += "&dn=";

            QString filename = QString(QUrl::toPercentEncoding(curitem->data(COLUMN_FILEBROWSER_NAME).toString()));

            filename.replace("%20", "+");
            all += filename;

            all += "\n";
        }

        // remove trailing "\n"
        all = all.trimmed();

        QApplication::clipboard()->setText(all);
    }        // Check if the user wants a non-default destination
    else if (chosen == dl_to_browse) {
        rootPath = QFileDialog::getExistingDirectory(this, tr("Select destination"));
        if (rootPath.isEmpty())
            return;

        QStringHash &hash = DCFileBrowser::GetDownloadDirs();

        if (!hash.contains(rootPath))
                hash.insert(rootPath, QDir(rootPath).dirName());
    }        // Should the file be renamed
    else if (chosen == dl_as) {
        QString save_root = DCConfig::Instance()->GetDownloadFinishedFolder().Data();

        if (save_root.isEmpty() || save_root.isNull())
            save_root = DCConfig::Instance()->GetDownloadFolder().Data();

        curitem = (FileBrowserItem*)selected.at(0).internalPointer();

        rootPath = QFileDialog::getSaveFileName(this, tr("Select a filename"),
                                                save_root + "/" + curitem->data(COLUMN_FILEBROWSER_NAME).toString(),
                                                tr("All files (*.*)"));
        if (rootPath.isEmpty())
            return;

        QFileInfo fi(rootPath);
        rootPath = fi.path();
        localName = fi.fileName();

        if (rootPath.isEmpty() || localName.isEmpty())
            return;
    } else if (chosen == dl_in) {
        QString localrootpath;
        QString localname;
        QString localpath;
        QString remotePath;

        if (!(curitem = (FileBrowserItem*)selected.at(0).internalPointer()))
            return;

        size = curitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong();
        QString tth = curitem->data(COLUMN_FILEBROWSER_TTH).toString();

        // all files need equal size and TTH
        for (int c = 1; c < selected.size(); c++) {

            curitem = (FileBrowserItem*)selected.at(c).internalPointer();

            if (size != curitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong())
                return;
            if (tth  != curitem->data(COLUMN_FILEBROWSER_TTH).toString())
                return;
        }

        if (!DCFileTool::SelectFileSource(this, size, tth, localname, localrootpath, localpath))
            return;

        QString remotePathAndFile;
        for (int c = 0; c < selected.size(); c++) {
            curitem = (FileBrowserItem*)selected.at(c).internalPointer();

            // get the full remote path
            remotePath = model->createRemotePath(curitem->parent());

            remotePathAndFile = model->createRemotePath(curitem);
            remotePathAndFile.replace("/", "\\");

            // add transfer to the waitlist
            DCFileTool::CheckFile(this, m_sNick.toAscii().constData(), m_sHubName.toAscii().constData(), m_sHubHost.toAscii().constData(),
                    remotePathAndFile.toAscii().constData(), localname.toAscii().constData(), localpath.toAscii().constData(), localrootpath.toAscii().constData(), eltFILE,
                    size, curitem->data(COLUMN_FILEBROWSER_TTH).toString().toAscii().constData(), true);
        }
    } else if (chosen == search_tth) {
        DCHubSearch * hubsearch = new DCHubSearch(DCConnectionManager::Instance()->GetMdiArea());

        connect(hubsearch, SIGNAL(onDie(QWidget*)), DCGuiApp::Instance(), SLOT(slotChildOnDie(QWidget*)));

        DCConnectionManager::Instance()->addTab((QWidget*)hubsearch,
                                                QApplication::translate("DCDialogHubSearch", "Hub Search", 0, QApplication::UnicodeUTF8));

        hubsearch->SetSearchForFile(curitem->data(COLUMN_FILEBROWSER_TTH).toString(), eftHASH);
        hubsearch->show();
        hubsearch->StartSearchWithPrompt();
    } else if (addedcommands.contains(chosen)) {
        DC_UserMenuCommand * umc = addedcommands[ chosen ];

        QString usercommand = umc->m_sCommand;
        QString origUserCommand = usercommand;
        QString fullfilename, filesize, filesizeshort, filetth, filetype;

        for (int c = 0; c < selected.size(); c++) {
            if (selected.at(c).column() != COLUMN_FILEBROWSER_NAME)
                continue;
            // filelist browsers are only for the one nick
            if ((umc->m_nType == euctRawOnce) && (c > 0))
                break;

            curitem = (FileBrowserItem*)selected.at(c).internalPointer();

            usercommand = client->replaceCommandTags(origUserCommand, m_sNick);

            if (usercommand.isEmpty()) {
                // had a %[line:reason] but dialog was cancelled
                continue;
            }

            if (direntry) {
                fullfilename = model->createRemotePath(curitem);
                filesize.setNum(curitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong());
                filesizeshort = curitem->data(COLUMN_FILEBROWSER_SIZE).toString();
                filetth = tr("None");
                filetype = tr("Directory");
            } else {
                fullfilename = model->createRemotePath(curitem->parent());
                filesize.setNum(curitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong());
                filesizeshort = curitem->data(COLUMN_FILEBROWSER_SIZE).toString();
                filetth = curitem->data(COLUMN_FILEBROWSER_TTH).toString();
                if (filetth.isEmpty()) {
                    filetth = tr("None");
                }

                if (curitem->isDir) {
                    filetype = tr("Directory");
                } else {
                    filetype = tr("File");
                    fullfilename += "\\";
                    fullfilename += curitem->data(COLUMN_FILEBROWSER_NAME).toString();
                }
            }

            usercommand.replace("%[file]", fullfilename);
            usercommand.replace("%[fileFN]", fullfilename);

            usercommand.replace("%[filesize]", filesize);
            usercommand.replace("%[fileSI]", filesize);

            usercommand.replace("%[filesizeshort]", filesizeshort);
            usercommand.replace("%[fileSIshort]", filesizeshort);

            usercommand.replace("%[tth]", filetth);
            usercommand.replace("%[fileTR]", filetth);

            usercommand.replace("%[type]", filetype);

            //client->m_pHubChat->AddStatus( usercommand.toAscii().constData() );
            client->SendString(usercommand.toAscii().constData());
        }
    }

    if (d_to_actions.contains(chosen)){
        if ((rootPath = t_tip) == "")
            return;
    }

    // Is this a download?
    if ((chosen == dl) || (chosen == dl_to_browse) || (chosen == dl_as) || t_tip != "") {
        for (int c = 0; c < selected.size(); c++) {
            curitem = (FileBrowserItem*)selected.at(c).internalPointer();

            // directory download from left pane
            if (direntry) {
                s = curitem->data(COLUMN_FILEBROWSER_NAME).toString();

                DownloadPath(rootPath, s, QString::null, curitem);
            } else if (curitem->isDir) // directory download from right pane
            {
                DownloadPath(rootPath, curitem->data(COLUMN_FILEBROWSER_NAME).toString(), QString::null, curitem);
            }                // Single file download
            else {
                QString remotePath;
                QString remoteName;

                // set localname
                if (chosen != dl_as) {
                    localName = curitem->data(COLUMN_FILEBROWSER_NAME).toString();
                }

                // get the full remote path
                remotePath = model->createRemotePath(curitem->parent());

                remoteName = curitem->data(COLUMN_FILEBROWSER_NAME).toString();

                DownloadFile(rootPath, QString(), localName, remotePath, remoteName,
                             curitem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong(),
                             curitem->data(COLUMN_FILEBROWSER_TTH).toString());
            }
        }
    }
}

/** */
void DCFileBrowser::DownloadPath(QString rootPath, QString localPath, QString localName, FileBrowserItem * item) {
    QString remotePath;
    QString s;
    QString remoteFile;
    FileBrowserItem *FileItem;

    if (!item || !item->isDir) {
        return;
    }

    // get the full remote path
    remotePath = model->createRemotePath(item);

    for (int i = 0; i < item->childCount(); i++) {
        FileItem = item->child(i);

        if (!FileItem->isDir) {
            remoteFile = FileItem->data(COLUMN_FILEBROWSER_NAME).toString();
            DownloadFile(rootPath, localPath, localName, remotePath,
                    remoteFile, FileItem->data(COLUMN_FILEBROWSER_ESIZE).toULongLong(), FileItem->data(COLUMN_FILEBROWSER_TTH).toString());
        }
    }

    item = (item->childCount() > 0)?item->child(0):NULL;

    while (item) {
        s = item->data(COLUMN_FILEBROWSER_NAME).toString();

        DownloadPath(rootPath, localPath + "\\" + s, localName, item);
        item = item->nextSibling();
    }
}

/** */
void DCFileBrowser::DownloadFile(QString rootPath, QString localPath, QString localName,
        QString remotePath, QString remoteName, ulonglong size,
        QString hash) {
    // append filename to the remote path
    remotePath += "\\";
    remotePath += remoteName;

    if (localName == QString::null) {
        localName = remoteName;
    }


    DCFileTool::CheckFile(this, m_sNick.toAscii().constData(),
            m_sHubName.toAscii().constData(),
            m_sHubHost.toAscii().constData(),
            remotePath.toAscii().constData(),
            localName.toAscii().constData(),
            localPath.toAscii().constData(),
            rootPath.toAscii().constData(), eltFILE,
            size, hash.toAscii().constData());
}

/** Double click to download the item, copied from DCFileBrowser::CreateMenu */
void DCFileBrowser::slotItemDoubleClicked(const QModelIndex &index) {

    if (!index.isValid())
        return;

    QString rootPath = QString::null;
    QString localName = QString::null;

    FileBrowserItem *item = static_cast<FileBrowserItem*>(index.internalPointer());

    if (!item)
        return;

    if (item->isDir){
        model->setRootElem(item, false);
    }
    else {
        QString remotePath;
        QString remoteName;

        // get the full remote path
        remotePath = model->createRemotePath(item->parent());

        remoteName = item->data(COLUMN_FILEBROWSER_NAME).toString();

        DownloadFile(rootPath, QString(), localName, remotePath, remoteName,
                     item->data(COLUMN_FILEBROWSER_ESIZE).toULongLong(),
                     item->data(COLUMN_FILEBROWSER_TTH).toString());
    }
}

/** Only needed for non-xml filelists */
ulonglong DCFileBrowser::CalcDirSize(FileBrowserItem * item) {
    ulonglong total = 0;

    QListIterator<FileBrowserItem*> it(item->childItems);

    FileBrowserItem* current = 0;

    while (it.hasNext()) {
        current = it.next();

        if (current->isDir) {
            FileBrowserItem * child = (item->childCount() > 0)?item->child(0):NULL;

            while (child) {
                if (child->data(COLUMN_FILEBROWSER_NAME).toString() == current->data(COLUMN_FILEBROWSER_NAME).toString()) {
                    total += CalcDirSize(child);
                    break;
                }

                child = child->nextSibling();
            }
        } else {
            total += current->data(COLUMN_FILEBROWSER_ESIZE).toULongLong();
        }
    }

    item->updateColumn(COLUMN_FILEBROWSER_ESIZE, total);
    item->updateColumn(COLUMN_FILEBROWSER_SIZE, DCGuiUtils::GetSizeString(total));

    //printf( "CalcDirSize for %s = %lld\n", item->text(0).toAscii().constData(), total );

    return total;
}

/** */
bool DCFileBrowser::DownloadDirectory(QString dir) {
    QStringList dirs = dir.split("\\", QString::SkipEmptyParts);

    qApp->processEvents();

    if (!model)
        return false;

    FileBrowserItem *curitem = model->createRootForPath(dir, rootElem);

    if (!curitem)
        return false;

    DownloadPath(QString::null, curitem->data(COLUMN_FILEBROWSER_NAME).toString(), QString::null, curitem);
    return true;
}

/** */
void DCFileBrowser::JumpTo(QString target) {
    FileBrowserItem *item = model->createRootForPath(target, rootElem);

    if (!item){
        m_sJumpTo = "";

        return;
    }
    m_sJumpTo = target;

    if (item->isDir)
        model->setRootElem(item, false, true);
    else
        model->setRootElem(item->parent(), false, true);

    QString path = model->createRemotePath(item);

    QModelIndex index = treeModel->createIndexForItem(treeModel->createRootForPath(path));

    if (!index.isValid())
        return;

    QStack<QModelIndex> stack;
    QModelIndex parent;

    stack.push(index);

    parent = index.parent();

    while (parent.isValid()){
        stack.push(parent);

        parent = parent.parent();
    }

    while (!stack.empty())
        treeView_DIRECTORY->expand(stack.pop());
    
    treeView_DIRECTORY->scrollTo(index);
    treeView_DIRECTORY->selectionModel()->setCurrentIndex(
        index, QItemSelectionModel::SelectCurrent);
}

/** */
void DCFileBrowser::slotGotoUser() {
    DCClient * client = 0;
    if (m_sNick.isEmpty() || m_sHubHost.isEmpty()) {
        client = 0;
    } else {
        client = DCConnectionManager::Instance()->GetClientForHub(m_sHubName.toAscii().constData(), m_sHubHost.toAscii().constData());
    }

    if (client) {
        if (client->jumpToNick(m_sNick)) {
            client->raise();
            client->setFocus();
        } else {
            QMessageBox::information(
                    this,
                    tr("Go to user"),
                    tr("Cannot find user")
                    );
        }
    } else {
        QMessageBox::information(
                this,
                tr("Go to user"),
                tr("Cannot find hub")
                );
    }
}

void DCFileBrowser::slotHistoryBack(){
    FileBrowserItem *item = history.prev();

    if (!item)
        return;

    history.isFlashBack = true;

    model->setRootElem(item, false, true);
}

void DCFileBrowser::slotHistoryForward(){
    FileBrowserItem *item = history.next();

    if (!item)
        return;

    model->setRootElem(item, false, true);
}

void DCFileBrowser::slotDirUp(){
    FileBrowserItem *root = model->getRootElem();

    if (root == rootElem || !root || !root->parent())
        return;

    model->setRootElem(root->parent(), false, true);
}

void DCFileBrowser::slotRootChanged(FileBrowserItem *from, FileBrowserItem *item){
    if (!item)
        return;

    if (from)
        history.add(from);

    toolButton_BACK->setEnabled(history.backSize() > 0);
    toolButton_FORW->setEnabled(history.forwardSize() > 0);
    toolButton_UP->setEnabled(item != rootElem);

    QString path = model->createRemotePath(item);

    path.replace("\\", "/");

    if (item->parent())
        lineEdit_PATH->setText(path);
    else{
        lineEdit_PATH->setText("");

        return;
    }

    model->sort(model->getSortColumn(), model->getSortOrder());

    QString proxy_path = model->createRemotePath(item);
    QModelIndex index = treeModel->createIndexForItem(treeModel->createRootForPath(proxy_path));

    if (!index.isValid())
        return;

    treeView_DIRECTORY->expand(index);
}

/** */
void DCFileBrowser::slotFILES_customContextMenuRequested(const QPoint &p){
    CreateMenu(p, false);
}

/** */
void DCFileBrowser::slotDIRECTORY_customContextMenuRequested(const QPoint &p){
    CreateMenu(p, true);
}

/** */
void DCFileBrowser::slotSearchEngineStateChanged(DCFileBrowserSearchEngine::State state){
    if (state == DCFileBrowserSearchEngine::RUN){
        lineEdit_SEARCHSTR->setEnabled(false);
        comboBox_SEARCHTYPE->setEnabled(false);
        pushButton_SEARCHRESET->setEnabled(false);

        pushButton_STARTSEARCH->setText(tr("Stop"));
    }
    else if (state == DCFileBrowserSearchEngine::DONE){
        lineEdit_SEARCHSTR->setEnabled(true);
        comboBox_SEARCHTYPE->setEnabled(true);
        pushButton_SEARCHRESET->setEnabled(true);

        pushButton_STARTSEARCH->setText(tr("Start"));
    }
    else {
        treeView_SEARCH->hide();

        lineEdit_SEARCHSTR->setEnabled(true);
        comboBox_SEARCHTYPE->setEnabled(true);
        pushButton_SEARCHRESET->setEnabled(true);

        pushButton_STARTSEARCH->setText(tr("Start"));
    }
}

/** */
void DCFileBrowser::slotDoSearch(){
    if (search_engine->state() == DCFileBrowserSearchEngine::RUN){
        search_engine->stop();
    }
    else {
        if (lineEdit_SEARCHSTR->text().isEmpty())
            return;

        DCFileBrowserSearchEngine::Type type = DCFileBrowserSearchEngine::All;

        switch (comboBox_SEARCHTYPE->currentIndex()){
            case 1:
                type = DCFileBrowserSearchEngine::Files;
                break;
            case 2:
                type = DCFileBrowserSearchEngine::Directories;
                break;
        }

        slotSearchReset();

        search_engine->setType(type);
        search_engine->setString(lineEdit_SEARCHSTR->text());

        search_engine->start();
    }
}

/** */
void DCFileBrowser::slotNewModel(FileBrowserModel *m){
    treeView_SEARCH->show();

    QAbstractItemModel * mod = treeView_SEARCH->model();

    treeView_SEARCH->setModel(m);

    delete mod;
}

/** */
void DCFileBrowser::slotNewElement(FileBrowserItem *){
    static int i = 0;
    i++;
    printf("+%i\n", i);
    treeView_FILES->repaint();
}

/** */
void DCFileBrowser::slotSearchReset(){
    treeView_SEARCH->setModel(new FileBrowserModel(this));

    treeView_SEARCH->hide();

    search_engine->reset();
}

void DCFileBrowser::closeEvent(QCloseEvent *e) {
    emit onDie(this);

    if (m_pContainer) {
        m_pContainer->setParent(0);
        m_pContainer->close();
    }

#if HAVE_MALLOC_TRIM
    malloc_trim(0);
#endif

    e->accept();
}
