/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dcfilebrowsersearchengine.h"
#include "filebrowsermodel.h"

#include <QMetaType>

DCFileBrowserSearchEngine::DCFileBrowserSearchEngine(FileBrowserModel *old_model):
        type(DCFileBrowserSearchEngine::All), src_str("")
{
    this->old_model = old_model;
    qRegisterMetaType<State>("DCFileBrowserSearchEngine::State");
    model = NULL;
    root = NULL;
}

DCFileBrowserSearchEngine::~DCFileBrowserSearchEngine()
{
    delete model;
}

void DCFileBrowserSearchEngine::run(){
    if (!old_model){
        emit done();

        emit stateChanged(engine_state = INT);

        if (model)
            model->setRootElem(NULL, false, false);

        delete model;

        model = NULL;

        return;
    }

    delete model;

    if (root)
        qDeleteAll(root->childItems);

    delete root;

    map.clear();

    QList<QVariant> rootData;
    rootData << tr("Name") << tr("Size") << tr("Exact size") << tr("Type") << tr("TTH");
    root= new FileBrowserItem(rootData, NULL);

    model = new FileBrowserModel(NULL);
    model->setIconsScaled(true, QSize(16, 16));
    model->setRootElem(root,false, true);

    emit modelCreated(model);

    emit stateChanged(engine_state = RUN);

    //printf("Begin search: %s - %i\n", src_str.toAscii().constData(), type);

    search(old_model->getRootElem()->childItems);

    emit done();

    emit stateChanged(engine_state = DONE);
}

void DCFileBrowserSearchEngine::search(QList<FileBrowserItem*> &list){
    bool include_dirs = (type == All) || (type == Directories);
    bool include_file = (type == Files) || (type == All);

    foreach (FileBrowserItem *i, list){
        bool contains = i->data(COLUMN_FILEBROWSER_NAME).toString().toUpper().indexOf(src_str.toUpper()) >= 0;
        bool ok = false;

        if (include_dirs && i->isDir)
            ok = true;
        else if (include_file && !i->isDir)
            ok = true;

        if (contains && ok){

            FileBrowserItem *item = new FileBrowserItem(QList<QVariant>(), root);
            *item = *i;

            map[item] = i;

            root->appendChild(item);

            emit newElement(item);
        }

        if (i->isDir)
            search(i->childItems);
    }
}

FileBrowserItem *DCFileBrowserSearchEngine::mapToOldModel(FileBrowserItem *item){
    if (map.contains(item))
        return map[item];

    return NULL;
}

void DCFileBrowserSearchEngine::setType(DCFileBrowserSearchEngine::Type t){
    type = t;
}

void DCFileBrowserSearchEngine::setString(const QString& s){
    src_str = s;
}

FileBrowserModel *DCFileBrowserSearchEngine::getOldModel(){
    return old_model;
}

DCFileBrowserSearchEngine::State DCFileBrowserSearchEngine::state() const{
    return engine_state;
}

void DCFileBrowserSearchEngine::stop(){
    emit stateChanged(engine_state = DONE);

    terminate();
}

void DCFileBrowserSearchEngine::reset(){
    if (isRunning())
        stop();

    if (model)
        model->setRootElem(NULL, false, false);

    delete model;

    model = NULL;

    delete root;

    root = NULL;
}
