#include "dcfilehasher.h"

#include <QFile>
#include <QTimer>
#include <QFileDialog>
#include <QUrl>
#include <QClipboard>
#include <QFileInfo>

#include <dclib/core/cbytearray.h>
#include <stdlib.h>
#include <stdint.h>

#include "dcconfig.h"
#include "dciconloader.h"

DCHashThread::DCHashThread(){
    file_hasher = NULL;
}

DCHashThread::~DCHashThread(){
    delete file_hasher;
}

void DCHashThread::run(){
    calculate_tth();
}

void DCHashThread::setFile(QString f){
    file_name = f;
}

QString DCHashThread::getHash(){
    return hash;
}

void DCHashThread::calculate_tth(){
    if (file_hasher){
        delete file_hasher;

        file_hasher = NULL;
    }

    if (!QFile::exists(file_name)){
        hash = "";

        return;
    }

    file_hasher = new CFileHasher(file_name.toAscii().constData());
    file_hasher->ComputeHash(efhsFinished);

    while (file_hasher->GetStatus() != efhsFinished);

    hash = file_hasher->GetHashRoot().Data();
}

DCFileHasher::DCFileHasher(QWidget *parent): QDialog(parent){
    setupUi(this);

    hasher = new DCHashThread();
    connect(hasher, SIGNAL(finished()), this, SLOT(slotDone()));

    initDocument();
}

DCFileHasher::~DCFileHasher(){
    if (hasher)
        hasher->terminate();

    delete hasher;
}

void DCFileHasher::initDocument(){
    pushButton_BROWSE->setIcon(DCIconLoader::Instance()->GetPixmap(eiFOLDER_BLUE_OPEN));

    connect(pushButton_RUN, SIGNAL(clicked()), this, SLOT(slotStart()));
    connect(pushButton_BROWSE, SIGNAL(clicked()), this, SLOT(slotBrowse()));
    connect(pushButton_MAGNET, SIGNAL(clicked()), this, SLOT(slotMagnet()));
}

void DCFileHasher::slotStart(){

    QString file = lineEdit_FILE->text();

    if (!QFile::exists(file))
        return;

    pushButton_RUN->setEnabled(false);

    if (!hasher){

    }
    else {
        hasher-> terminate();
    }

    hasher->setFile(file);

    hasher->start();
}

void DCFileHasher::slotDone(){
    lineEdit_HASH->setText(hasher->getHash());

    pushButton_RUN->setEnabled(true);
}

void DCFileHasher::slotMagnet(){    
    QString magnet = "", tth = lineEdit_HASH->text(), file = lineEdit_FILE->text();

    if (tth.isEmpty())
        return;

    magnet += "magnet:?xt=urn:tree:tiger:" + tth;
    magnet += "&xl=" + QString().setNum(QFile(file).size());

    QStringList list = file.split("/");

    file = list.last();

    if (file.isEmpty())
        return;

    file = QUrl::toPercentEncoding(file);
    file.replace("%20", "+");

    magnet += "&dn=" + file;

    QApplication::clipboard()->setText(magnet);
}

void DCFileHasher::slotBrowse(){
    QString directory = QString::fromUtf8(DCConfig::Instance()->GetDownloadFinishedFolder().Data());

    if (directory.isEmpty() || directory.isNull() || !QDir(directory).exists())
        directory = QString::fromUtf8(DCConfig::Instance()->GetDownloadFolder().Data());

    QString file = QFileDialog::getOpenFileName(this, tr("Select file"), directory, tr("All files (*.*)"));

    if (!file.isEmpty()){
        lineEdit_FILE->setText(file);
        lineEdit_HASH->setText("");
    }
}
